!-------------------- BEGIN small_integrate.f90 BEGIN ---------------------
! @file small_integrate.f90                                                
! @author jlinford                                                         
! @date 2012-06-22 15:43:23.334090                                         
! @brief Interface to time stepping integrator                             
!                                                                          
! Definitions of interface functions for the Kppa-generated                
! time stepping integrator.  These are the Kppa "entry point" routines.    
!                                                                          
! This file was generated by Kppa: http://www.paratools.com/Kppa           
!--------------------------------------------------------------------------


MODULE small_integrate

    USE small_parameters
    USE small_rosenbrock

    IMPLICIT NONE





    CONTAINS

!----------------------------- GridIntegrate ------------------------------
! Applies the Kppa-generated integrator to the grid                        
!                                                                          
! @param[in]     ncells Number of grid cells                               
! @param[in,out] conc   Species concentrations                             
! @param[in]     tstart Integration start time                             
! @param[in]     tend   Integration end time                               
! @param[in]     abstol Absolute integration tolerances for variable species 
! @param[in]     reltol Relative integration tolerances for variable species 
! @param[in,out] idata  Integer integration in/out parameters              
! @param[in,out] rdata  Real value integration in/out parameters           
! @param[out]    lastH  Last timestep in each grid cell                    
!--------------------------------------------------------------------------
    SUBROUTINE GridIntegrate(ncells, conc, tstart, tend, abstol, reltol, &
            idata, rdata, lastH)
        IMPLICIT NONE

        INTEGER, INTENT(IN) :: ncells
        REAL(4), INTENT(INOUT) :: conc(NVAR*ncells)
        REAL(4), INTENT(IN) :: tstart
        REAL(4), INTENT(IN) :: tend
        REAL(4), INTENT(IN) :: abstol(5)
        REAL(4), INTENT(IN) :: reltol(5)
        INTEGER, INTENT(INOUT) :: idata(20)
        REAL(4), INTENT(INOUT) :: rdata(20)
        REAL(4), INTENT(OUT) :: lastH(ncells)

    INTEGER :: var
    INTEGER :: fix
    INTEGER :: i

    DO i = 1, ncells
        var = (i-1)*NSPEC + 1
        fix = var + NVAR

        ! Invoke the integrator
        CALL Integrate(conc(var), conc(fix), i, tstart, tend, &
                       abstol, reltol, idata, rdata)

        ! Save the last timestep for future use
        lastH(i) = rdata(12)

        ! Report status
        IF (idata(20) /= 0) THEN
            WRITE(*,*) "Kppa: INTEGRATION FAILED IN CELL",i
            WRITE(*,*) "Kppa: idata=",idata
            WRITE(*,*) "Kppa: rdata=",rdata
            RETURN
        END IF
    END DO
    END SUBROUTINE GridIntegrate


END MODULE small_integrate
!---------------------- END small_integrate.f90 END -----------------------
